@@ -10,21 +10,33 @@ from os import PathLike
 from typing import BinaryIO, Iterator, List, Tuple
 
 from .oxml import (
+    Bullet,
     HwpxOxmlDocument,
     HwpxOxmlHeader,
+    HwpxOxmlHistory,
     HwpxOxmlInlineObject,
+    HwpxOxmlMasterPage,
     HwpxOxmlMemo,
     HwpxOxmlParagraph,
     HwpxOxmlRun,
     HwpxOxmlSection,
+    HwpxOxmlSectionHeaderFooter,
     HwpxOxmlTable,
+    HwpxOxmlVersion,
     MemoShape,
+    ParagraphProperty,
     RunStyle,
+    Style,
+    TrackChange,
+    TrackChangeAuthor,
 )
 from .package import HwpxPackage
+from .templates import blank_document_bytes
 
 _HP_NS = "http://www.hancom.co.kr/hwpml/2011/paragraph"
 _HP = f"{{{_HP_NS}}}"
+_HH_NS = "http://www.hancom.co.kr/hwpml/2011/head"
+_HH = f"{{{_HH_NS}}}"
 
 
 class HwpxDocument:
@@ -46,6 +58,12 @@ class HwpxDocument:
         root = HwpxOxmlDocument.from_package(package)
         return cls(package, root)
 
+    @classmethod
+    def new(cls) -> "HwpxDocument":
+        """Return a new blank document based on the default skeleton template."""
+
+        return cls.open(blank_document_bytes())
+
     @classmethod
     def from_package(cls, package: HwpxPackage) -> "HwpxDocument":
         """Create a document backed by an existing :class:`HwpxPackage`."""
@@ -74,6 +92,21 @@ class HwpxDocument:
         """Return the header parts referenced by the document."""
         return self._root.headers
 
+    @property
+    def master_pages(self) -> List[HwpxOxmlMasterPage]:
+        """Return the master-page parts declared in the manifest."""
+        return self._root.master_pages
+
+    @property
+    def histories(self) -> List[HwpxOxmlHistory]:
+        """Return document history parts referenced by the manifest."""
+        return self._root.histories
+
+    @property
+    def version(self) -> HwpxOxmlVersion | None:
+        """Return the version metadata part if present."""
+        return self._root.version
+
     @property
     def memo_shapes(self) -> dict[str, MemoShape]:
         """Return memo shapes available in the header reference lists."""
@@ -85,6 +118,65 @@ class HwpxDocument:
 
         return self._root.memo_shape(memo_shape_id_ref)
 
+    @property
+    def bullets(self) -> dict[str, Bullet]:
+        """Return bullet definitions declared in header reference lists."""
+
+        return self._root.bullets
+
+    def bullet(self, bullet_id_ref: int | str | None) -> Bullet | None:
+        """Return the bullet definition referenced by *bullet_id_ref*."""
+
+        return self._root.bullet(bullet_id_ref)
+
+    @property
+    def paragraph_properties(self) -> dict[str, ParagraphProperty]:
+        """Return paragraph property definitions declared in headers."""
+
+        return self._root.paragraph_properties
+
+    def paragraph_property(
+        self, para_pr_id_ref: int | str | None
+    ) -> ParagraphProperty | None:
+        """Return the paragraph property referenced by *para_pr_id_ref*."""
+
+        return self._root.paragraph_property(para_pr_id_ref)
+
+    @property
+    def styles(self) -> dict[str, Style]:
+        """Return style definitions available in the document."""
+
+        return self._root.styles
+
+    def style(self, style_id_ref: int | str | None) -> Style | None:
+        """Return the style definition referenced by *style_id_ref*."""
+
+        return self._root.style(style_id_ref)
+
+    @property
+    def track_changes(self) -> dict[str, TrackChange]:
+        """Return tracked change metadata declared in the headers."""
+
+        return self._root.track_changes
+
+    def track_change(self, change_id_ref: int | str | None) -> TrackChange | None:
+        """Return tracked change metadata referenced by *change_id_ref*."""
+
+        return self._root.track_change(change_id_ref)
+
+    @property
+    def track_change_authors(self) -> dict[str, TrackChangeAuthor]:
+        """Return tracked change author metadata declared in the headers."""
+
+        return self._root.track_change_authors
+
+    def track_change_author(
+        self, author_id_ref: int | str | None
+    ) -> TrackChangeAuthor | None:
+        """Return tracked change author details referenced by *author_id_ref*."""
+
+        return self._root.track_change_author(author_id_ref)
+
     @property
     def memos(self) -> List[HwpxOxmlMemo]:
         """Return all memo entries declared in every section."""
@@ -295,6 +387,23 @@ class HwpxDocument:
 
         return self._root.char_property(char_pr_id_ref)
 
+    def ensure_run_style(
+        self,
+        *,
+        bold: bool = False,
+        italic: bool = False,
+        underline: bool = False,
+        base_char_pr_id: str | int | None = None,
+    ) -> str:
+        """Return a ``charPr`` identifier matching the requested flags."""
+
+        return self._root.ensure_run_style(
+            bold=bold,
+            italic=italic,
+            underline=underline,
+            base_char_pr_id=base_char_pr_id,
+        )
+
     def iter_runs(self) -> Iterator[HwpxOxmlRun]:
         """Yield every run element contained in the document."""
 
@@ -363,11 +472,17 @@ class HwpxDocument:
                 remaining = limit - replacements
                 if remaining <= 0:
                     break
-            replacements += run.replace_text(
+            original_char_pr = run.char_pr_id_ref
+            replaced_here = run.replace_text(
                 search,
                 replacement,
                 count=remaining,
             )
+            if replaced_here and original_char_pr is not None:
+                # Ensure the run retains its original formatting reference even
+                # if XML nodes were rewritten during substitution.
+                run.char_pr_id_ref = original_char_pr
+            replacements += replaced_here
             if limit is not None and replacements >= limit:
                 break
         return replacements
@@ -507,6 +622,80 @@ class HwpxDocument:
             char_pr_id_ref=char_pr_id_ref,
         )
 
+    def set_header_text(
+        self,
+        text: str,
+        *,
+        section: HwpxOxmlSection | None = None,
+        section_index: int | None = None,
+        page_type: str = "BOTH",
+    ) -> HwpxOxmlSectionHeaderFooter:
+        """Ensure the requested section contains a header for *page_type* and set its text."""
+
+        target_section = section
+        if target_section is None and section_index is not None:
+            target_section = self._root.sections[section_index]
+        if target_section is None:
+            if not self._root.sections:
+                raise ValueError("document does not contain any sections")
+            target_section = self._root.sections[-1]
+        return target_section.properties.set_header_text(text, page_type=page_type)
+
+    def set_footer_text(
+        self,
+        text: str,
+        *,
+        section: HwpxOxmlSection | None = None,
+        section_index: int | None = None,
+        page_type: str = "BOTH",
+    ) -> HwpxOxmlSectionHeaderFooter:
+        """Ensure the requested section contains a footer for *page_type* and set its text."""
+
+        target_section = section
+        if target_section is None and section_index is not None:
+            target_section = self._root.sections[section_index]
+        if target_section is None:
+            if not self._root.sections:
+                raise ValueError("document does not contain any sections")
+            target_section = self._root.sections[-1]
+        return target_section.properties.set_footer_text(text, page_type=page_type)
+
+    def remove_header(
+        self,
+        *,
+        section: HwpxOxmlSection | None = None,
+        section_index: int | None = None,
+        page_type: str = "BOTH",
+    ) -> None:
+        """Remove the header linked to *page_type* from the requested section if present."""
+
+        target_section = section
+        if target_section is None and section_index is not None:
+            target_section = self._root.sections[section_index]
+        if target_section is None:
+            if not self._root.sections:
+                return
+            target_section = self._root.sections[-1]
+        target_section.properties.remove_header(page_type=page_type)
+
+    def remove_footer(
+        self,
+        *,
+        section: HwpxOxmlSection | None = None,
+        section_index: int | None = None,
+        page_type: str = "BOTH",
+    ) -> None:
+        """Remove the footer linked to *page_type* from the requested section if present."""
+
+        target_section = section
+        if target_section is None and section_index is not None:
+            target_section = self._root.sections[section_index]
+        if target_section is None:
+            if not self._root.sections:
+                return
+            target_section = self._root.sections[-1]
+        target_section.properties.remove_footer(page_type=page_type)
+
     def save(
         self,
         path_or_stream: str | PathLike[str] | BinaryIO | None = None,
